@@ -317,7 +317,7 @@ class fock_space_kron:
             s = list(state)
             return(self.f1.index(state[:self.f1.modes]) * self.f2.dimension + self.f2.index(state[self.f1.modes:]))
         
-def real_time_solver(psi0, dt, tmax, H, Q = None, final_state = None):
+    def real_time_solver(psi0, dt, tmax, H, Q = None, final_state = None, H_int = None):
   
     K = psi0.size
     Nt = int(tmax/dt)+1
@@ -327,60 +327,118 @@ def real_time_solver(psi0, dt, tmax, H, Q = None, final_state = None):
         H_const = H
         def H(t):
             return(H_const)
-     
-    if Q == None:
-        for i in range(1,Nt):
-            psi_iter_old = psi0
-            psi_iter = np.zeros(K)
-            psi_compare = psi0
-            while (LA.norm(psi_iter-psi_compare)>10**(-6)):
-                s = psi[1]+psi_iter_old
-                psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
-                psi_compare = psi_iter_old
-                psi_iter_old = psi_iter
-            psi[1] = psi_iter
-            
-        return (psi[1])
         
-    elif type(Q) == list:
-        l = len(Q)
-        results= np.zeros((l,Nt))
-        for j in range(l):
-            results[j,0] = abs(np.conj(psi0) @ Q[j] @ psi0)
         
-        for i in range(1,Nt):
-            psi_iter_old = psi0
-            psi_iter = np.zeros(K)
-            psi_compare = psi0
-            while (LA.norm(psi_iter-psi_compare)>10**(-6)):
-                s = psi[1]+psi_iter_old
-                psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
-                psi_compare = psi_iter_old
-                psi_iter_old = psi_iter
-            psi[1] = psi_iter
+    if H_int = None:
+        
+        if Q == None:
+            for i in range(1,Nt):
+                psi_iter_old = psi0
+                psi_iter = np.zeros(K)
+                psi_compare = psi0
+                while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                    s = psi[1]+psi_iter_old
+                    psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
+                    psi_compare = psi_iter_old
+                    psi_iter_old = psi_iter
+                psi[1] = psi_iter
+                
+            return (psi[1])
             
+        elif type(Q) == list:
+            l = len(Q)
+            results= np.zeros((l,Nt))
             for j in range(l):
-                results[j,i] = abs(np.conj(psi[1]) @ Q[j] @ psi[1])
+                results[j,0] = abs(np.conj(psi0) @ Q[j] @ psi0)
+            
+            for i in range(1,Nt):
+                psi_iter_old = psi0
+                psi_iter = np.zeros(K)
+                psi_compare = psi0
+                while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                    s = psi[1]+psi_iter_old
+                    psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
+                    psi_compare = psi_iter_old
+                    psi_iter_old = psi_iter
+                psi[1] = psi_iter
+                
+                for j in range(l):
+                    results[j,i] = abs(np.conj(psi[1]) @ Q[j] @ psi[1])
+            
+        else:
+            results = np.zeros(Nt)
+            results[0] = abs(np.conj(psi0) @ Q @ psi0)
+            
+            for i in range(1,Nt):
+                psi_iter_old = psi0
+                psi_iter = np.zeros(K)
+                psi_compare = psi0
+                while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                    s = psi[1]+psi_iter_old
+                    psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
+                    psi_compare = psi_iter_old
+                    psi_iter_old = psi_iter
+                psi[1] = psi_iter
+                results[i] = abs(np.conj(psi[1]) @ Q @ psi[1])
+        if final_state != None :
+            return (results, psi[1])
+        else:
+            return(results)
         
     else:
-        results = np.zeros(Nt)
-        results[0] = abs(np.conj(psi0) @ Q @ psi0)
+        if Q == None:
+            for i in range(1,Nt):
+                psi_iter_old = psi0
+                psi_iter = np.zeros(K)
+                psi_compare = psi0
+                while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                    s = psi[1]+psi_iter_old
+                    psi_iter = psi[1]+ dt/2*(-1j) * H(i+1/2)@s
+                    psi_compare = psi_iter_old
+                    psi_iter_old = psi_iter
+                psi[1] = psi_iter
+                
+            return (psi[1])
+            
+        elif type(Q) == list:
+            l = len(Q)
+            results= np.zeros((l,Nt))
+            for j in range(l):
+                results[j,0] = abs(np.conj(psi0) @ Q[j] @ psi0)
+            
+            for i in range(1,Nt):
+                psi_iter_old = psi0
+                psi_iter = np.zeros(K)
+                psi_compare = psi0
+                while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                    s = psi[1]+psi_iter_old
+                    psi_iter = psi[1]+ dt/2*(-1j) * H(i+1/2)@s
+                    psi_compare = psi_iter_old
+                    psi_iter_old = psi_iter
+                psi[1] = psi_iter
+                
+                for j in range(l):
+                    results[j,i] = abs(np.conj(psi[1]) @ Q[j] @ psi[1])
+            
+        else:
+            results = np.zeros(Nt)
+            results[0] = abs(np.conj(psi0) @ Q @ psi0)
+            
+            for i in range(1,Nt):
+                psi_iter_old = psi0
+                psi_iter = np.zeros(K)
+                psi_compare = psi0
+                while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                    s = psi[1]+psi_iter_old
+                    psi_iter = psi[1]+ dt/2*(-1j) * H(i+1/2)@s
+                    psi_compare = psi_iter_old
+                    psi_iter_old = psi_iter
+                psi[1] = psi_iter
+                results[i] = abs(np.conj(psi[1]) @ Q @ psi[1])
+        if final_state != None :
+            return (results, psi[1])
+        else:
+            return(results)
         
-        for i in range(1,Nt):
-            psi_iter_old = psi0
-            psi_iter = np.zeros(K)
-            psi_compare = psi0
-            while (LA.norm(psi_iter-psi_compare)>10**(-6)):
-                s = psi[1]+psi_iter_old
-                psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
-                psi_compare = psi_iter_old
-                psi_iter_old = psi_iter
-            psi[1] = psi_iter
-            results[i] = abs(np.conj(psi[1]) @ Q @ psi[1])
-    if final_state != None :
-        return (results, psi[1])
-    else:
-        return(results)
-    
         
 
